import socket
import threading
import time

UDP_IP = "255.255.255.255"
UDP_PORT = 5005
LISTEN_PORT = 5006

def broadcast_presence():
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sock.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)

    while True:
        message = b"ROBOBIN_PRESENT"
        sock.sendto(message, (UDP_IP, UDP_PORT))
        print("Broadcasting: {}".format(message.decode()))
        time.sleep(5)

def handle_ping(client_socket):
    print("Received PING from client.")
    response = b"PONG"
    print("Sending PONG to client.")
    client_socket.sendall(response)

def handle_time_request(client_socket):
    current_time = time.ctime().encode()
    print(f"Sending current time: {current_time.decode()}")
    client_socket.sendall(current_time)

def handle_custom_message(client_socket, message):
    response = f"Received custom message: {message}".encode()
    print(f"Custom handler response: {response.decode()}")
    client_socket.sendall(response)
def handle_unknown_message(client_socket):
    response = b"I don't know this message."
    print("Sending response to unknown message.")
    client_socket.sendall(response)

message_handlers = {
    "PING": handle_ping,
    "TIME": handle_time_request,
    "CUSTOM": handle_custom_message,  
}

def handle_client_connection(client_socket):
    try:
        while True:
            request = client_socket.recv(1024)
            if not request:
                print("No request received, closing connection.")
                break 
            message = request.decode()
            print("Received from client: {}".format(message))

  
            parts = message.split(" ", 1)
            message_type = parts[0]
            message_data = parts[1] if len(parts) > 1 else None

            if message_type in message_handlers:
                if message_type == "CUSTOM" and message_data:
                    message_handlers[message_type](client_socket, message_data)
                else:
                    message_handlers[message_type](client_socket)
            else:
                handle_unknown_message(client_socket)

    except ConnectionResetError:
        print("Client connection was forcibly closed.")
    except Exception as e:
        print(f"An error occurred while handling the client connection: {e}")
    finally:
        client_socket.close()
        print("Client disconnected.")

def listen_for_connections():
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sock.bind(('', LISTEN_PORT))

    while True:
        try:
            data, addr = sock.recvfrom(1024)
            if data.decode() == "CONNECT":
                print("Received connection request from {}".format(addr))

                with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as tcp_socket:
                    tcp_socket.bind(('', 5006)) 
                    tcp_socket.listen(1)
                    print("Listening for TCP connection...")

                    client_socket, client_addr = tcp_socket.accept()
                    print("Client connected from {}".format(client_addr))

                    threading.Thread(target=handle_client_connection, args=(client_socket,)).start()
        except Exception as e:
            print(f"An error occurred while listening for connections: {e}")

# Start broadcasting and listening threads
broadcast_thread = threading.Thread(target=broadcast_presence)
listen_thread = threading.Thread(target=listen_for_connections)

broadcast_thread.start()
listen_thread.start()

broadcast_thread.join()
listen_thread.join()
